@@ -23,8 +23,20 @@ extensions = [
     'sphinxcontrib.apidoc',
     'stevedore.sphinxext',
     'openstackdocstheme',
+    'oslo_config.sphinxext',
+    'oslo_config.sphinxconfiggen',
+    'oslo_policy.sphinxext',
+    'oslo_policy.sphinxpolicygen',
 ]
 
+config_generator_config_file = '../../etc/config-generator.conf'
+sample_config_basename = '_static/tacker'
+
+policy_generator_config_file = [
+    ('../../etc/tacker-policy-generator.conf', '_static/tacker'),
+]
+sample_policy_basename = '_static/tacker'
+
 # sphinxcontrib.apidoc options
 apidoc_module_dir = '../../tacker'
 apidoc_output_dir = 'contributor/api'
@@ -37,6 +49,9 @@ apidoc_excluded_paths = [
     'vnfm/workflows',
     'db/migration/alembic_migrations/*',
     'db/migration/alembic_migrations',
+    # TODO(ueha): Excluding 'db/types.py' will be removed after the issue of
+    # TypeDecorator is fixed.
+    'db/types.py',
     'hacking']
 apidoc_separate_modules = True
 
@@ -47,8 +62,8 @@ source_suffix = '.rst'
 master_doc = 'index'
 
 # General information about the project.
-project = u'tacker'
-copyright = u'2013, OpenStack Foundation'
+project = 'tacker'
+copyright = '2013, OpenStack Foundation'
 
 # If true, '()' will be appended to :func: etc. cross-reference text.
 add_function_parentheses = True
@@ -58,7 +73,7 @@ add_function_parentheses = True
 add_module_names = True
 
 # The name of the Pygments (syntax highlighting) style to use.
-pygments_style = 'sphinx'
+pygments_style = 'native'
 
 # A list of ignored prefixes for module index sorting.
 modindex_common_prefix = ['tacker.']
@@ -77,25 +92,21 @@ htmlhelp_basename = '%sdoc' % project
 # robots.txt.
 html_extra_path = ['_extra']
 
-# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
-# using the given strftime format.
-# html_last_updated_fmt = '%b %d, %Y'
-html_last_updated_fmt = '%Y-%m-%d %H:%M'
-
 # Grouping the document tree into LaTeX files. List of tuples
 # (source start file, target name, title, author, documentclass
 # [howto/manual]).
 latex_documents = [
     ('index',
      '%s.tex' % project,
-     u'%s Documentation' % project,
-     u'OpenStack Foundation', 'manual'),
+     '%s Documentation' % project,
+     'OpenStack Foundation', 'manual'),
 ]
 
 # Example configuration for intersphinx: refer to the Python standard library.
 #intersphinx_mapping = {'http://docs.python.org/': None}
 
 # -- Options for openstackdocstheme -------------------------------------------
-repository_name = 'openstack/tacker'
-bug_project = 'tacker'
-bug_tag = ''
+openstackdocs_repo_name = 'openstack/tacker'
+openstackdocs_auto_name = False
+openstackdocs_bug_project = 'tacker'
+openstackdocs_bug_tag = ''
