@@ -56,8 +56,11 @@ from tacker import manager
 from tacker import objects
 from tacker.objects import fields
 from tacker.objects.fields import ErrorPoint as EP
+from tacker.objects.vnf_instance import VnfInstanceList as vnf_instance_list
 from tacker.objects import vnf_lcm_op_occs as vnf_lcm_op_occs_obj
+from tacker.objects.vnf_lcm_op_occs import VnfLcmOpOccList as vnf_lcm_op_list
 from tacker.objects import vnf_lcm_subscriptions as subscription_obj
+from tacker.objects.vnf_lcm_subscriptions import LccnSubscriptionList as s_list
 from tacker.plugins.common import constants
 from tacker.policies import vnf_lcm as vnf_lcm_policies
 from tacker.tosca import utils as toscautils
@@ -192,9 +195,6 @@ class VnfLcmController(wsgi.Controller):
         self._vnfm_plugin = manager.TackerManager.get_service_plugins()['VNFM']
         self._view_builder_op_occ = vnf_op_occs_view.ViewBuilder()
         self._view_builder_subscription = vnf_subscription_view.ViewBuilder()
-        self._nextpages_vnf_instances = {}
-        self._nextpages_lcm_op_occs = {}
-        self._nextpages_subscriptions = {}
 
     def _get_vnf_instance_href(self, vnf_instance):
         return '{}vnflcm/v1/vnf_instances/{}'.format(
@@ -288,7 +288,7 @@ class VnfLcmController(wsgi.Controller):
         vim_client_obj = vim_client.VimClient()
 
         try:
-            vim_client_obj.get_vim(
+            return vim_client_obj.get_vim(
                 context, vim_id, region_name=region_name)
         except nfvo.VimDefaultNotDefined as exp:
             raise webob.exc.HTTPBadRequest(explanation=exp.message)
@@ -439,7 +439,8 @@ class VnfLcmController(wsgi.Controller):
     @validation.schema(vnf_lcm.create)
     def create(self, request, body):
         context = request.environ['tacker.context']
-        context.can(vnf_lcm_policies.VNFLCM % 'create')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_lcm_policies.VNFLCM % 'create')
         try:
             req_body = utils.convert_camelcase_to_snakecase(body)
             vnfd_id = req_body.get('vnfd_id')
@@ -466,6 +467,10 @@ class VnfLcmController(wsgi.Controller):
                     vnfd_id)
                 return self._make_problem_detail(
                     msg, 500, 'Internal Server Error')
+        if CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_lcm_policies.VNFLCM % 'create',
+                        target={'vendor': vnfd.vnf_provider})
+
         try:
             # get default vim information
             vim_client_obj = vim_client.VimClient()
@@ -489,6 +494,17 @@ class VnfLcmController(wsgi.Controller):
             try:
                 vnf_instance.create()
 
+                vim_conn = objects.VimConnectionInfo(
+                    id=uuidutils.generate_uuid(),
+                    vim_id=default_vim.get('vim_id'),
+                    vim_type=default_vim.get('vim_type'),
+                    access_info={},
+                    interface_info={},
+                    extra=default_vim.get('extra', {})
+                )
+                vnf_instance.vim_connection_info = [vim_conn]
+                vnf_instance.save()
+
                 # create entry to 'vnf' table and 'vnf_attribute' table
                 attributes = {'placement_attr': default_vim.
                     get('placement_attr', {})}
@@ -560,19 +576,44 @@ class VnfLcmController(wsgi.Controller):
 
         return vnf_package_info[0]
 
-    def _delete_expired_nextpages(self, nextpages):
-        for k, v in list(nextpages.items()):
-            if timeutils.is_older_than(v['created_time'],
-                    CONF.vnf_lcm.nextpage_expiration_time):
-                LOG.debug('Old nextpages are deleted. id: %s' % k)
-                nextpages.pop(k)
+    def _get_policy_target(self, vnf_instance):
+        vendor = vnf_instance.vnf_provider
+
+        area = None
+        if vnf_instance.vim_connection_info:
+            for connection_info in vnf_instance.vim_connection_info:
+                area = connection_info.extra.get('area')
+                if area:
+                    break
+
+        tenant = None
+        if (vnf_instance.instantiation_state ==
+                fields.VnfInstanceState.INSTANTIATED):
+            if vnf_instance.vnf_metadata:
+                tenant = vnf_instance.vnf_metadata.get('tenant')
+        else:
+            tenant = '*'
+
+        target = {
+            'vendor': vendor,
+            'area': area,
+            'tenant': tenant
+        }
+
+        target = {k: v for k, v in target.items() if v is not None}
+
+        return target
 
     @wsgi.response(http_client.OK)
     @wsgi.expected_errors((http_client.FORBIDDEN, http_client.NOT_FOUND))
     def show(self, request, id):
         context = request.environ['tacker.context']
-        context.can(vnf_lcm_policies.VNFLCM % 'show')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_lcm_policies.VNFLCM % 'show')
         vnf_instance = self._get_vnf_instance(context, id)
+        if CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_lcm_policies.VNFLCM % 'show',
+                        target=self._get_policy_target(vnf_instance))
 
         return self._view_builder.show(vnf_instance)
 
@@ -586,8 +627,8 @@ class VnfLcmController(wsgi.Controller):
     @api_common.validate_supported_params({'filter', 'nextpage_opaque_marker',
                                            'all_records'})
     def index(self, request):
-        if 'tacker.context' in request.environ:
-            context = request.environ['tacker.context']
+        context = request.environ['tacker.context']
+        if not CONF.oslo_policy.enhanced_tacker_policy:
             context.can(vnf_lcm_policies.VNFLCM % 'index')
 
         filters = request.GET.get('filter')
@@ -596,39 +637,50 @@ class VnfLcmController(wsgi.Controller):
         nextpage = request.GET.get('nextpage_opaque_marker')
         allrecords = request.GET.get('all_records')
 
+        limit = None
+        marker_obj = None
         result = []
 
-        if allrecords != 'yes' and nextpage:
-            self._delete_expired_nextpages(self._nextpages_vnf_instances)
+        # get maximum record size per page
+        if allrecords != 'yes':
+            limit = CONF.vnf_lcm.vnf_instance_num
 
-            if nextpage in self._nextpages_vnf_instances:
-                result = self._nextpages_vnf_instances.pop(
-                    nextpage)['nextpage']
-        else:
-            vnf_instances = objects.VnfInstanceList.get_by_filters(
-                request.context, filters=filters)
+            # get next page marker object from nextpage id
+            if nextpage:
+                marker_obj = objects.VnfInstance.get_by_id(request.context,
+                                                           nextpage)
+        try:
+            # get records from DB within maximum record size per page
+            # except for getting all records case
+            result = vnf_instance_list.get_by_marker_filter(request.context,
+                    limit, marker_obj, filters=filters)
+        except Exception as e:
+            LOG.exception(traceback.format_exc())
+            return self._make_problem_detail(
+                str(e), 500, title='Internal Server Error')
 
-            result = self._view_builder.index(vnf_instances)
+        if CONF.oslo_policy.enhanced_tacker_policy:
+            result = [vnf_instance for vnf_instance in result
+                if context.can(
+                    vnf_lcm_policies.VNFLCM % 'index',
+                    target=self._get_policy_target(vnf_instance),
+                    fatal=False)]
+
+        result = self._view_builder.index(result)
 
         res = webob.Response(content_type='application/json')
         res.status_int = 200
 
-        if (allrecords != 'yes' and
-                len(result) > CONF.vnf_lcm.vnf_instance_num):
-            nextpageid = uuidutils.generate_uuid()
+        # if the number of records obtained from DB is equal to maximum record
+        # size per page, the id of the last record is used as next page marker
+        # and set it to Link header of the response
+        if (allrecords != 'yes' and len(result) >= limit):
+            nextpageid = result[(limit - 1)]['id']
             links = ('Link', '<%s?nextpage_opaque_marker=%s>; rel="next"' % (
                 request.path_url, nextpageid))
             res.headerlist.append(links)
-            res.body = jsonutils.dump_as_bytes(
-                result[: CONF.vnf_lcm.vnf_instance_num])
 
-            self._delete_expired_nextpages(self._nextpages_vnf_instances)
-
-            remain = result[CONF.vnf_lcm.vnf_instance_num:]
-            self._nextpages_vnf_instances.update({nextpageid:
-                {'created_time': timeutils.utcnow(), 'nextpage': remain}})
-        else:
-            res.body = jsonutils.dump_as_bytes(result)
+        res.body = jsonutils.dump_as_bytes(result)
 
         return res
 
@@ -653,6 +705,10 @@ class VnfLcmController(wsgi.Controller):
         context = request.environ['tacker.context']
 
         vnf_instance = self._get_vnf_instance(context, id)
+        if CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_lcm_policies.VNFLCM % 'delete',
+                        target=self._get_policy_target(vnf_instance))
+
         vnf = self._get_vnf(context, id)
         self._delete(context, vnf_instance, vnf)
 
@@ -686,7 +742,12 @@ class VnfLcmController(wsgi.Controller):
                 req_body, context=context)
 
         # validate the vim connection id passed through request is exist or not
-        self._validate_vim_connection(context, instantiate_vnf_request)
+        vim_res = self._validate_vim_connection(
+            context, instantiate_vnf_request)
+
+        if instantiate_vnf_request.vim_connection_info:
+            instantiate_vnf_request.vim_connection_info[0].extra = vim_res.get(
+                'extra', {})
 
         vnf_instance.task_state = fields.VnfInstanceTaskState.INSTANTIATING
         vnf_instance.save()
@@ -718,10 +779,14 @@ class VnfLcmController(wsgi.Controller):
     @validation.schema(vnf_lcm.instantiate)
     def instantiate(self, request, id, body):
         context = request.environ['tacker.context']
-        context.can(vnf_lcm_policies.VNFLCM % 'instantiate')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_lcm_policies.VNFLCM % 'instantiate')
 
         vnf = self._get_vnf(context, id)
         vnf_instance = self._get_vnf_instance(context, id)
+        if CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_lcm_policies.VNFLCM % 'instantiate',
+                        target=self._get_policy_target(vnf_instance))
 
         return self._instantiate(context, vnf_instance, vnf, body)
 
@@ -769,10 +834,14 @@ class VnfLcmController(wsgi.Controller):
     @validation.schema(vnf_lcm.terminate)
     def terminate(self, request, id, body):
         context = request.environ['tacker.context']
-        context.can(vnf_lcm_policies.VNFLCM % 'terminate')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_lcm_policies.VNFLCM % 'terminate')
 
         vnf = self._get_vnf(context, id)
         vnf_instance = self._get_vnf_instance(context, id)
+        if CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_lcm_policies.VNFLCM % 'terminate',
+                        target=self._get_policy_target(vnf_instance))
         return self._terminate(context, vnf_instance, vnf, body)
 
     @check_vnf_status_and_error_point(action="heal",
@@ -825,10 +894,14 @@ class VnfLcmController(wsgi.Controller):
     @validation.schema(vnf_lcm.heal)
     def heal(self, request, id, body):
         context = request.environ['tacker.context']
-        context.can(vnf_lcm_policies.VNFLCM % 'heal')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_lcm_policies.VNFLCM % 'heal')
 
         vnf = self._get_vnf(context, id)
         vnf_instance = self._get_vnf_instance(context, id)
+        if CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_lcm_policies.VNFLCM % 'heal',
+                        target=self._get_policy_target(vnf_instance))
 
         if vnf_instance.instantiation_state not in \
            [fields.VnfInstanceState.INSTANTIATED]:
@@ -868,7 +941,12 @@ class VnfLcmController(wsgi.Controller):
     @wsgi.expected_errors((http_client.FORBIDDEN, http_client.NOT_FOUND))
     def update(self, request, id, body):
         context = request.environ['tacker.context']
-        context.can(vnf_lcm_policies.VNFLCM % 'update_vnf')
+        if CONF.oslo_policy.enhanced_tacker_policy:
+            vnf_instance = self._get_vnf_instance(context, id)
+            context.can(vnf_lcm_policies.VNFLCM % 'update_vnf',
+                        target=self._get_policy_target(vnf_instance))
+        else:
+            context.can(vnf_lcm_policies.VNFLCM % 'update_vnf')
 
         # get body
         body_data = {}
@@ -1061,9 +1139,9 @@ class VnfLcmController(wsgi.Controller):
     @wsgi.response(http_client.OK)
     def subscription_list(self, request):
         nextpage_opaque_marker = None
-        paging = 1
         filter_string = ""
-        ignore_nextpages = False
+        limit = None
+        marker_obj = None
         subscription_data = []
 
         query_params = request.query_string
@@ -1083,9 +1161,6 @@ class VnfLcmController(wsgi.Controller):
                     filter_string = query_param_key_value[1]
                 if query_param_key_value[0] == 'nextpage_opaque_marker':
                     nextpage_opaque_marker = query_param_key_value[1]
-                if query_param_key_value[0] == 'page':
-                    paging = int(query_param_key_value[1])
-                    ignore_nextpages = True
 
             if filter_string:
                 # check enumerations columns
@@ -1114,55 +1189,48 @@ class VnfLcmController(wsgi.Controller):
                                 title='Bad Request')
 
         nextpage = nextpage_opaque_marker
-        if allrecords != 'yes' and not ignore_nextpages and nextpage:
-            self._delete_expired_nextpages(self._nextpages_subscriptions)
 
-            if nextpage in self._nextpages_subscriptions:
-                subscription_data = self._nextpages_subscriptions.pop(
-                    nextpage)['nextpage']
-        else:
-            try:
-                filter_string_parsed = self._view_builder_subscription. \
-                    validate_filter(filter_string)
-                if nextpage_opaque_marker:
-                    start_index = paging - 1
-                else:
-                    start_index = None
+        try:
+            filter_string_parsed = self._view_builder_subscription. \
+                validate_filter(filter_string)
 
-                vnf_lcm_subscriptions = (
-                    subscription_obj.LccnSubscriptionList.
-                    get_by_filters(request.context,
-                                read_deleted='no',
-                                filters=filter_string_parsed,
-                                nextpage_opaque_marker=start_index))
+            # get maximum record size per page
+            if allrecords != 'yes':
+                limit = CONF.vnf_lcm.subscription_num
 
-                LOG.debug("vnf_lcm_subscriptions %s" % vnf_lcm_subscriptions)
-                subscription_data = self._view_builder_subscription. \
-                    subscription_list(vnf_lcm_subscriptions)
-            except Exception as e:
-                LOG.error(traceback.format_exc())
-                return self._make_problem_detail(
-                    str(e), 500, title='Internal Server Error')
+            # get next page marker object from nextpage id
+            if nextpage:
+                marker_obj = objects.LccnSubscription.get_by_id(
+                    request.context, nextpage)
+
+            # get records from DB within maximum record size per page
+            # except for getting all records case
+            result = s_list.get_by_marker_filter(
+                request.context, limit,
+                marker_obj, filters=filter_string_parsed,
+                read_deleted='no')
+            LOG.debug("vnf_lcm_subscriptions %s" % result)
+
+            subscription_data = self._view_builder_subscription. \
+                subscription_list(result)
+        except Exception as e:
+            LOG.error(traceback.format_exc())
+            return self._make_problem_detail(
+                str(e), 500, title='Internal Server Error')
 
         # make response
         res = webob.Response(content_type='application/json')
         res.status_int = 200
 
-        if (allrecords != 'yes' and not ignore_nextpages and
-                len(subscription_data) > CONF.vnf_lcm.subscription_num):
-            nextpageid = uuidutils.generate_uuid()
+        # if the number of records obtained from DB is equal to maximum record
+        # size per page, the id of the last record is used as next page marker
+        # and set it to Link header of the response
+        if (allrecords != 'yes' and len(subscription_data) >= limit):
+            nextpageid = subscription_data[(limit - 1)]['id']
             links = ('Link', '<%s?nextpage_opaque_marker=%s>; rel="next"' % (
                 request.path_url, nextpageid))
             res.headerlist.append(links)
 
-            remain = subscription_data[CONF.vnf_lcm.subscription_num:]
-            subscription_data = (
-                subscription_data[: CONF.vnf_lcm.subscription_num])
-
-            self._delete_expired_nextpages(self._nextpages_subscriptions)
-            self._nextpages_subscriptions.update({nextpageid:
-                {'created_time': timeutils.utcnow(), 'nextpage': remain}})
-
         res.body = jsonutils.dump_as_bytes(subscription_data)
 
         LOG.debug("subscription_list res %s" % res)
@@ -1188,8 +1256,9 @@ class VnfLcmController(wsgi.Controller):
         vnfd_dict = vnflcm_utils._get_vnfd_dict(context,
             vnf_instance.vnfd_id,
             vnf_instance.instantiated_vnf_info.flavour_id)
-        tosca = tosca_template.ToscaTemplate(parsed_params={}, a_file=False,
-                                             yaml_dict_tpl=vnfd_dict)
+        tosca = tosca_template.ToscaTemplate(
+            parsed_params={}, a_file=False, yaml_dict_tpl=vnfd_dict,
+            local_defs=toscautils.tosca_tmpl_local_defs())
         tosca_policies = tosca.topology_template.policies
 
         aspect_max_level_dict = {}
@@ -1356,7 +1425,8 @@ class VnfLcmController(wsgi.Controller):
                            http_client.NOT_FOUND, http_client.CONFLICT))
     def scale(self, request, id, body):
         context = request.environ['tacker.context']
-        context.can(vnf_lcm_policies.VNFLCM % 'scale')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_lcm_policies.VNFLCM % 'scale')
 
         try:
             vnf_info = self._vnfm_plugin.get_vnf(context, id)
@@ -1364,6 +1434,9 @@ class VnfLcmController(wsgi.Controller):
                 return self._make_problem_detail(
                     'VNF IS NOT ACTIVE', 409, title='VNF IS NOT ACTIVE')
             vnf_instance = self._get_vnf_instance(context, id)
+            if CONF.oslo_policy.enhanced_tacker_policy:
+                context.can(vnf_lcm_policies.VNFLCM % 'scale',
+                            target=self._get_policy_target(vnf_instance))
             if not vnf_instance.instantiated_vnf_info.scale_status:
                 return self._make_problem_detail(
                     'NOT SCALE VNF', 409, title='NOT SCALE VNF')
@@ -1374,6 +1447,8 @@ class VnfLcmController(wsgi.Controller):
         except webob.exc.HTTPNotFound as inst_e:
             return self._make_problem_detail(
                 str(inst_e), 404, title='VNF NOT FOUND')
+        except exceptions.PolicyNotAuthorized:
+            raise
         except Exception as e:
             LOG.error(traceback.format_exc())
             return self._make_problem_detail(
@@ -1748,52 +1823,53 @@ class VnfLcmController(wsgi.Controller):
         nextpage = request.GET.get('nextpage_opaque_marker')
         allrecords = request.GET.get('all_records')
 
+        limit = None
+        marker_obj = None
         result = []
 
-        if allrecords != 'yes' and nextpage:
-            self._delete_expired_nextpages(self._nextpages_lcm_op_occs)
+        self._view_builder_op_occ.validate_attribute_fields(
+            all_fields=all_fields, fields=fields,
+            exclude_fields=exclude_fields,
+            exclude_default=exclude_default)
 
-            if nextpage in self._nextpages_lcm_op_occs:
-                result = self._nextpages_lcm_op_occs.pop(nextpage)['nextpage']
-        else:
-            self._view_builder_op_occ.validate_attribute_fields(
-                all_fields=all_fields, fields=fields,
-                exclude_fields=exclude_fields,
-                exclude_default=exclude_default)
+        filters = self._view_builder_op_occ.validate_filter(filters)
 
-            filters = self._view_builder_op_occ.validate_filter(filters)
+        # get maximum record size per page
+        if allrecords != 'yes':
+            limit = CONF.vnf_lcm.lcm_op_occ_num
 
-            try:
-                vnf_lcm_op_occs = (
-                    vnf_lcm_op_occs_obj.VnfLcmOpOccList.get_by_filters(
-                        request.context, read_deleted='no', filters=filters))
-            except Exception as e:
-                LOG.exception(traceback.format_exc())
-                return self._make_problem_detail(
-                    str(e), 500, title='Internal Server Error')
+        # get next page marker object from nextpage id
+        if nextpage:
+            marker_obj = vnf_lcm_op_occs_obj.VnfLcmOpOcc.get_by_id(
+                request.context, nextpage)
 
-            result = self._view_builder_op_occ.index(request, vnf_lcm_op_occs,
-                    all_fields=all_fields, exclude_fields=exclude_fields,
-                    fields=fields, exclude_default=exclude_default)
+        try:
+            # get records from DB within maximum record size per page
+            # except for getting all records case
+            result = vnf_lcm_op_list.get_by_marker_filter(request.context,
+                    limit, marker_obj, filters=filters, read_deleted='no')
+        except Exception as e:
+            LOG.exception(traceback.format_exc())
+            return self._make_problem_detail(
+                str(e), 500, title='Internal Server Error')
+
+        result = self._view_builder_op_occ.index(request, result,
+             all_fields=all_fields, exclude_fields=exclude_fields,
+             fields=fields, exclude_default=exclude_default)
 
         res = webob.Response(content_type='application/json')
         res.status_int = 200
 
-        if allrecords != 'yes' and len(result) > CONF.vnf_lcm.lcm_op_occ_num:
-            nextpageid = uuidutils.generate_uuid()
+        # if the number of records obtained from DB is equal to maximum record
+        # size per page, the id of the last record is used as next page marker
+        # and set it to Link header of the response
+        if (allrecords != 'yes' and len(result) >= limit):
+            nextpageid = result[(limit - 1)]['id']
             links = ('Link', '<%s?nextpage_opaque_marker=%s>; rel="next"' % (
                 request.path_url, nextpageid))
             res.headerlist.append(links)
-            res.body = jsonutils.dump_as_bytes(
-                result[: CONF.vnf_lcm.lcm_op_occ_num])
 
-            self._delete_expired_nextpages(self._nextpages_lcm_op_occs)
-
-            remain = result[CONF.vnf_lcm.lcm_op_occ_num:]
-            self._nextpages_lcm_op_occs.update({nextpageid:
-                {'created_time': timeutils.utcnow(), 'nextpage': remain}})
-        else:
-            res.body = jsonutils.dump_as_bytes(result)
+        res.body = jsonutils.dump_as_bytes(result)
 
         return res
 
@@ -1831,10 +1907,14 @@ class VnfLcmController(wsgi.Controller):
     @validation.schema(vnf_lcm.change_ext_conn)
     def change_ext_conn(self, request, id, body):
         context = request.environ['tacker.context']
-        context.can(vnf_lcm_policies.VNFLCM % 'change_ext_conn')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_lcm_policies.VNFLCM % 'change_ext_conn')
 
         vnf = self._get_vnf(context, id)
         vnf_instance = self._get_vnf_instance(context, id)
+        if CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_lcm_policies.VNFLCM % 'change_ext_conn',
+                        target=self._get_policy_target(vnf_instance))
         if (vnf_instance.instantiation_state !=
                 fields.VnfInstanceState.INSTANTIATED):
             return self._make_problem_detail(
@@ -1842,7 +1922,7 @@ class VnfLcmController(wsgi.Controller):
                 409,
                 title='VNF IS NOT INSTANTIATED')
         vnf['before_error_point'] = EP.INITIAL
-        self._change_ext_conn(context, vnf_instance, vnf, body)
+        return self._change_ext_conn(context, vnf_instance, vnf, body)
 
     def _change_ext_conn(self, context, vnf_instance, vnf, request_body):
         req_body = utils.convert_camelcase_to_snakecase(request_body)
@@ -1868,6 +1948,14 @@ class VnfLcmController(wsgi.Controller):
             change_ext_conn_req,
             vnf_lcm_op_occs_id)
 
+        # set response header
+        res = webob.Response()
+        res.status_int = 202
+        location = ('Location',
+            self._get_vnf_lcm_op_occs_href(vnf_lcm_op_occs_id))
+        res.headerlist.append(location)
+        return res
+
 
 def create_resource():
     return wsgi.Resource(VnfLcmController())
