@@ -18,6 +18,7 @@ from io import BytesIO
 import json
 import mimetypes
 import os
+import traceback
 import webob
 import zipfile
 from zipfile import ZipFile
@@ -25,6 +26,7 @@ from zipfile import ZipFile
 from glance_store import exceptions as store_exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import uuidutils
@@ -38,8 +40,10 @@ from tacker.common import exceptions
 from tacker.common import utils
 from tacker.conductor.conductorrpc import vnf_pkgm_rpc
 from tacker.glance_store import store as glance_store
+from tacker import objects
 from tacker.objects import fields
 from tacker.objects import vnf_package as vnf_package_obj
+from tacker.objects.vnf_package import VnfPackagesList as vnf_package_list
 from tacker.policies import vnf_package as vnf_package_policies
 from tacker import wsgi
 
@@ -57,6 +61,7 @@ class VnfPkgmController(wsgi.Controller):
         super(VnfPkgmController, self).__init__()
         self.rpc_api = vnf_pkgm_rpc.VNFPackageRPCAPI()
         glance_store.initialize_glance_store()
+        self._nextpages = {}
 
     def _get_vnf_package(self, id, request):
         # check if id is of type uuid format
@@ -99,7 +104,8 @@ class VnfPkgmController(wsgi.Controller):
     @wsgi.expected_errors((http_client.FORBIDDEN, http_client.NOT_FOUND))
     def show(self, request, id):
         context = request.environ['tacker.context']
-        context.can(vnf_package_policies.VNFPKGM % 'show')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_package_policies.VNFPKGM % 'show')
 
         # check if id is of type uuid format
         if not uuidutils.is_uuid_like(id):
@@ -110,18 +116,34 @@ class VnfPkgmController(wsgi.Controller):
             vnf_package = vnf_package_obj.VnfPackage.get_by_id(
                 request.context, id, expected_attrs=[
                     "vnf_deployment_flavours", "vnfd", "vnf_artifacts"])
+            if CONF.oslo_policy.enhanced_tacker_policy:
+                context.can(vnf_package_policies.VNFPKGM % 'show',
+                            target=self._get_policy_target(vnf_package))
         except exceptions.VnfPackageNotFound:
             msg = _("Can not find requested vnf package: %s") % id
             raise webob.exc.HTTPNotFound(explanation=msg)
 
         return self._view_builder.show(vnf_package)
 
+    def _get_policy_target(self, vnf_package):
+        if vnf_package.onboarding_state == \
+                fields.PackageOnboardingStateType.ONBOARDED:
+
+            vendor = (vnf_package.vnfd.get('vnf_provider')
+                      if vnf_package.vnfd is not None else None)
+
+            return {'vendor': vendor} if vendor else {}
+
+        else:
+            return {'vendor': '*'}
+
     @wsgi.response(http_client.OK)
     @wsgi.expected_errors((http_client.BAD_REQUEST, http_client.FORBIDDEN))
     @validation.query_schema(vnf_packages.query_params_v1)
     def index(self, request):
         context = request.environ['tacker.context']
-        context.can(vnf_package_policies.VNFPKGM % 'index')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_package_policies.VNFPKGM % 'index')
 
         search_opts = {}
         search_opts.update(request.GET)
@@ -139,6 +161,8 @@ class VnfPkgmController(wsgi.Controller):
         fields = request.GET.get('fields')
         exclude_fields = request.GET.get('exclude_fields')
         filters = request.GET.get('filter')
+        nextpage = request.GET.get('nextpage_opaque_marker')
+        allrecords = request.GET.get('all_records')
         if not (all_fields or fields or exclude_fields):
             exclude_default = True
 
@@ -148,23 +172,70 @@ class VnfPkgmController(wsgi.Controller):
 
         filters = self._view_builder.validate_filter(filters)
 
-        vnf_packages = vnf_package_obj.VnfPackagesList.get_by_filters(
-            request.context, read_deleted='no', filters=filters)
+        results = []
+        limit = None
+        marker_obj = None
 
-        return self._view_builder.index(vnf_packages,
-                                        all_fields=all_fields,
-                                        exclude_fields=exclude_fields,
-                                        fields=fields,
-                                        exclude_default=exclude_default)
+        if allrecords != 'yes':
+            limit = CONF.vnf_package.vnf_package_num
+
+            # get next page marker object from nextpage id
+            if nextpage:
+                marker_obj = objects.VnfPackage.get_by_id(request.context,
+                        nextpage)
+
+        try:
+            # get records from DB within maximum record size per page
+            # except for getting all records case
+            result = vnf_package_list.get_by_marker_filter(request.context,
+                    limit, marker_obj, filters=filters, read_deleted='no')
+        except Exception as e:
+            LOG.exception(traceback.format_exc())
+            return self._make_problem_detail(
+                str(e), 500, title='Internal Server Error')
+
+        if CONF.oslo_policy.enhanced_tacker_policy:
+            result = [vnf_package for vnf_package in result
+                      if context.can(
+                          vnf_package_policies.VNFPKGM % 'index',
+                          target=self._get_policy_target(vnf_package),
+                          fatal=False)]
+
+        results = self._view_builder.index(result,
+                all_fields=all_fields,
+                exclude_fields=exclude_fields,
+                fields=fields,
+                exclude_default=exclude_default)
+
+        res = webob.Response(content_type='application/json')
+        res.status_int = 200
+
+        # if the number of records obtained from DB is equal to maximum record
+        # size per page, the id of the last record is used as next page marker
+        # and set it to Link header of the response
+        if (allrecords != 'yes' and
+                len(results) >= limit):
+            nextpageid = result[(limit - 1)]['id']
+            links = ('Link', '<%s?nextpage_opaque_marker=%s>; rel="next"' % (
+                request.path_url, nextpageid))
+            res.headerlist.append(links)
+
+        res.body = jsonutils.dump_as_bytes(results, default=str)
+
+        return res
 
     @wsgi.response(http_client.NO_CONTENT)
     @wsgi.expected_errors((http_client.FORBIDDEN, http_client.NOT_FOUND,
                            http_client.CONFLICT))
     def delete(self, request, id):
         context = request.environ['tacker.context']
-        context.can(vnf_package_policies.VNFPKGM % 'delete')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_package_policies.VNFPKGM % 'delete')
 
         vnf_package = self._get_vnf_package(id, request)
+        if CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_package_policies.VNFPKGM % 'delete',
+                        target=self._get_policy_target(vnf_package))
 
         if (vnf_package.operational_state ==
                 fields.PackageOperationalStateType.ENABLED or
@@ -179,8 +250,20 @@ class VnfPkgmController(wsgi.Controller):
                     "operational_state": vnf_package.operational_state,
                     "usage_state": vnf_package.usage_state})
 
-        # Delete vnf_package
-        self.rpc_api.delete_vnf_package(context, vnf_package)
+        # Controller deletes vnf_package just from DB here.
+        # Related objects in Legacy DB are also deleted.
+        # Actual vnf_packages that is in backend storage
+        # or each conductors' local filesystems are deleted after.
+        if vnf_package.vnfd is not None:
+            objects.VnfdAttribute(context).delete(vnf_package.vnfd.vnfd_id)
+            objects.Vnfd(context).delete(vnf_package.vnfd.vnfd_id)
+        vnf_package.destroy(context)
+
+        if vnf_package.onboarding_state == (
+                fields.PackageOnboardingStateType.ONBOARDED):
+            # send rpc message for deleting actual vnf_package
+            # to conductors
+            self.rpc_api.delete_vnf_package(context, vnf_package)
 
     @wsgi.response(http_client.ACCEPTED)
     @wsgi.expected_errors((http_client.FORBIDDEN, http_client.NOT_FOUND,
@@ -188,9 +271,13 @@ class VnfPkgmController(wsgi.Controller):
                            http_client.REQUESTED_RANGE_NOT_SATISFIABLE))
     def fetch_vnf_package_content(self, request, id):
         context = request.environ['tacker.context']
-        context.can(vnf_package_policies.VNFPKGM % 'fetch_package_content')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_package_policies.VNFPKGM % 'fetch_package_content')
 
         vnf_package = self._get_vnf_package(id, request)
+        if CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_package_policies.VNFPKGM % 'fetch_package_content',
+                        target=self._get_policy_target(vnf_package))
 
         if vnf_package.onboarding_state != \
            fields.PackageOnboardingStateType.ONBOARDED:
@@ -297,6 +384,13 @@ class VnfPkgmController(wsgi.Controller):
                        "[0, %s].") % (zip_file_size - 1)
                 raise webob.exc.HTTPRequestRangeNotSatisfiable(
                     explanation=msg)
+            if range_.end is not None:
+                if range_.end - 1 >= zip_file_size:
+                    msg = _("Invalid end position in Range header. "
+                           "End position MUST be in the inclusive range"
+                           "[0, %d].") % (zip_file_size - 1)
+                    raise webob.exc.HTTPRequestRangeNotSatisfiable(
+                        explanation=msg)
             return range_
 
     @wsgi.response(http_client.ACCEPTED)
@@ -304,7 +398,9 @@ class VnfPkgmController(wsgi.Controller):
                            http_client.CONFLICT))
     def upload_vnf_package_content(self, request, id, body):
         context = request.environ['tacker.context']
-        context.can(vnf_package_policies.VNFPKGM % 'upload_package_content')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(
+                vnf_package_policies.VNFPKGM % 'upload_package_content')
 
         # check if id is of type uuid format
         if not uuidutils.is_uuid_like(id):
@@ -341,6 +437,13 @@ class VnfPkgmController(wsgi.Controller):
         try:
             (location, size, checksum, multihash,
             loc_meta) = glance_store.store_csar(context, id, body)
+            if CONF.oslo_policy.enhanced_tacker_policy:
+                zip_path = glance_store.load_csar(vnf_package.id, location)
+                vnf_data, flavours, vnf_artifacts = csar_utils.load_csar_data(
+                    context.elevated(), vnf_package.id, zip_path)
+                context.can(
+                    vnf_package_policies.VNFPKGM % 'upload_package_content',
+                    target={'vendor': vnf_data.get('provider')})
         except exceptions.UploadFailedToGlanceStore:
             with excutils.save_and_reraise_exception():
                 vnf_package.onboarding_state = (
@@ -350,6 +453,19 @@ class VnfPkgmController(wsgi.Controller):
                 except Exception as e:
                     return self._make_problem_detail(
                         'Internal Server Error', str(e), 500)
+        except exceptions.Forbidden:
+            with excutils.save_and_reraise_exception():
+                vnf_package.onboarding_state = (
+                    fields.PackageOnboardingStateType.CREATED)
+                try:
+                    # Delete from glance store
+                    glance_store.delete_csar(context, vnf_package.id,
+                                             location)
+                    csar_utils.delete_csar_data(vnf_package.id)
+                    vnf_package.save()
+                except Exception as e:
+                    return self._make_problem_detail(
+                        'Internal Server Error', str(e), 500)
 
         vnf_package.algorithm = CONF.vnf_package.hashing_algorithm
         vnf_package.hash = multihash
@@ -412,9 +528,13 @@ class VnfPkgmController(wsgi.Controller):
     @validation.schema(vnf_packages.patch)
     def patch(self, request, id, body):
         context = request.environ['tacker.context']
-        context.can(vnf_package_policies.VNFPKGM % 'patch')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_package_policies.VNFPKGM % 'patch')
 
         old_vnf_package = self._get_vnf_package(id, request)
+        if CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_package_policies.VNFPKGM % 'patch',
+                        target=self._get_policy_target(old_vnf_package))
         vnf_package = old_vnf_package.obj_clone()
 
         user_data = body.get('userDefinedData')
@@ -466,7 +586,8 @@ class VnfPkgmController(wsgi.Controller):
                            http_client.INTERNAL_SERVER_ERROR))
     def get_vnf_package_vnfd(self, request, id):
         context = request.environ['tacker.context']
-        context.can(vnf_package_policies.VNFPKGM % 'get_vnf_package_vnfd')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_package_policies.VNFPKGM % 'get_vnf_package_vnfd')
 
         valid_accept_headers = ['application/zip', 'text/plain']
         accept_headers = request.headers['Accept'].split(',')
@@ -480,6 +601,9 @@ class VnfPkgmController(wsgi.Controller):
                                            valid_accept_headers)})
 
         vnf_package = self._get_vnf_package(id, request)
+        if CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_package_policies.VNFPKGM % 'get_vnf_package_vnfd',
+                        target=self._get_policy_target(vnf_package))
 
         if vnf_package.onboarding_state != \
                 fields.PackageOnboardingStateType.ONBOARDED:
@@ -523,7 +647,8 @@ class VnfPkgmController(wsgi.Controller):
     def fetch_vnf_package_artifacts(self, request, id, artifact_path):
         context = request.environ['tacker.context']
         # get policy
-        context.can(vnf_package_policies.VNFPKGM % 'fetch_artifact')
+        if not CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(vnf_package_policies.VNFPKGM % 'fetch_artifact')
 
         # get vnf_package
         if not uuidutils.is_uuid_like(id):
@@ -534,6 +659,10 @@ class VnfPkgmController(wsgi.Controller):
             vnf_package = vnf_package_obj.VnfPackage.get_by_id(
                 request.context, id,
                 expected_attrs=["vnf_artifacts"])
+            if CONF.oslo_policy.enhanced_tacker_policy:
+                # get policy
+                context.can(vnf_package_policies.VNFPKGM % 'fetch_artifact',
+                            target=self._get_policy_target(vnf_package))
         except exceptions.VnfPackageNotFound:
             msg = _("Can not find requested vnf package: %s") % id
             raise webob.exc.HTTPNotFound(explanation=msg)
