@@ -87,6 +87,17 @@ else:
     getargspec = inspect.getargspec
 
 
+def proj_root():
+    """Return tacker's project root dir.
+
+    It's assumed `abspath(__file__)` is equals to ${PROJ_ROOT}/tacker/common.
+    The result is like as "/opt/stack/tacker".
+    """
+
+    dir_to_common = os.path.dirname(os.path.abspath(__file__))
+    return "/".join(dir_to_common.split('/')[:-2])
+
+
 def find_config_file(options, config_file):
     """Return the first config file found.
 
@@ -174,6 +185,18 @@ def is_valid_ipv4(address):
         return False
 
 
+def is_valid_area(area):
+    """Verify that the area attribute is valid.
+
+    Area attribute is an area-region pair. The value of this attribute should
+    be a string in the format of "area@region".
+    """
+    try:
+        return re.match(r'\w+@\w+', area) is not None
+    except Exception:
+        return False
+
+
 def change_memory_unit(mem, to):
     """Change the memory value(mem) based on the unit('to') specified.
 
@@ -355,13 +378,15 @@ def convert_camelcase_to_snakecase(request_data):
     This method takes care only keys in a `dict` or `dicts in a list`.
     For simple list with string items, the elements which are actual values
     are ignored during conversion.
+    And property value is not converted when key is `additionalParams`.
 
     :param request_data: dict with keys or list with items, in camelCase.
     """
     if isinstance(request_data, dict):
         new_dict = {}
         for key, property_value in request_data.items():
-            property_value = convert_camelcase_to_snakecase(property_value)
+            if key != "additionalParams":
+                property_value = convert_camelcase_to_snakecase(property_value)
             underscore_joined = convert_string_to_snakecase(key)
             new_dict[underscore_joined] = property_value
         return new_dict
@@ -386,6 +411,7 @@ def convert_snakecase_to_camelcase(request_data):
     Also, Snake case is a notation method that uses underscores to connect
     words. For that reason, if the initial word of the key in dict starts
     with '_', this function ignore to convert the key.
+    And property value is not converted when key is `additional_params`.
 
     :param request_data: dict with keys or list with items, in snake_case.
 
@@ -419,7 +445,8 @@ def convert_snakecase_to_camelcase(request_data):
     if isinstance(request_data, dict):
         new_dict = {}
         for key, property_value in request_data.items():
-            property_value = convert_snakecase_to_camelcase(property_value)
+            if key != "additional_params":
+                property_value = convert_snakecase_to_camelcase(property_value)
             camelcase = key if key.startswith('_') else convert(key)
             new_dict[camelcase] = property_value
         return new_dict
@@ -618,8 +645,8 @@ class MemoryUnit(object):
             unit = MemoryUnit.validate_unit(unit)
         else:
             unit = MemoryUnit.UNIT_SIZE_DEFAULT
-            LOG.info(_('A memory unit is not provided for size; using the '
-                       'default unit %(default)s.') % {'default': 'B'})
+            LOG.debug(_('A memory unit is not provided for size; using the '
+                        'default unit %(default)s.') % {'default': 'B'})
         result = re.sub(r'\s+', ' ', size).split(' ')
         if len(result) == 2:
             if result[1]:
@@ -628,9 +655,9 @@ class MemoryUnit(object):
                                 MemoryUnit.UNIT_SIZE_DICT[unit_size] *
                                 math.pow(MemoryUnit.UNIT_SIZE_DICT
                                          [unit], -1))
-                LOG.info(_('Given size %(size)s is converted to %(num)s '
-                           '%(unit)s.') % {'size': size,
-                         'num': converted, 'unit': unit})
+                LOG.debug(_('Given size %(size)s is converted to %(num)s '
+                            '%(unit)s.') % {'size': size,
+                            'num': converted, 'unit': unit})
             else:
                 msg = _('Size is not given for software image data.')
                 LOG.error(msg)
@@ -680,3 +707,13 @@ def str_to_bytes(value):
         value = bytes(value)
 
     return value
+
+
+def str_to_bool(value):
+    """Convert string to Bool"""
+    if value == 'True':
+        return True
+    if value == 'False':
+        return False
+    msg = _('Can\'t convert "{}" to Bool'.format(value))
+    raise ValueError(msg)
