@@ -29,6 +29,7 @@ from tacker.common import utils
 from tacker.db import api as db_api
 from tacker.db.db_sqlalchemy import api
 from tacker.db.db_sqlalchemy import models
+from tacker.db import sqlalchemyutils
 from tacker import objects
 from tacker.objects import base
 from tacker.objects import common
@@ -115,6 +116,39 @@ def _update_user_defined_data(context, package_uuid, user_data):
     return result
 
 
+@db_api.context_manager.reader
+def _vnf_packages_get_by_filters_query(context, read_deleted=None,
+                                    filters=None):
+
+    query = api.model_query(context, models.VnfPackage,
+                            read_deleted=read_deleted,
+                            project_only=True).options(joinedload('_metadata'))
+
+    if filters:
+        # Need to join VnfDeploymentFlavour, VnfSoftwareImage and
+        # VnfSoftwareImageMetadata db table explicitly
+        # only when filters contains one of the column matching
+        # from VnfSoftwareImage or VnfSoftwareImageMetadata db table.
+        filter_data = json.dumps(filters)
+        if 'VnfSoftwareImageMetadata' in filter_data:
+            query = query.join(models.VnfDeploymentFlavour).join(
+                models.VnfSoftwareImage).join(
+                models.VnfSoftwareImageMetadata)
+        elif 'VnfSoftwareImage' in filter_data:
+            query = query.join(models.VnfDeploymentFlavour).join(
+                models.VnfSoftwareImage)
+
+        if 'VnfPackageArtifactInfo' in filter_data:
+            query = query.join(models.VnfPackageArtifactInfo)
+
+        if 'VnfPackageVnfd' in filter_data:
+            query = query.join(models.VnfPackageVnfd)
+
+        query = common.apply_filters(query, filters)
+
+    return query
+
+
 @db_api.context_manager.reader
 def _vnf_package_get_by_id(context, package_uuid, columns_to_join=None):
 
@@ -346,7 +380,8 @@ class VnfPackage(base.TackerObject, base.TackerPersistentObject,
         'vnfd': fields.ObjectField('VnfPackageVnfd', nullable=True),
         'size': fields.IntegerField(nullable=False, default=0),
         'vnf_artifacts': fields.ObjectField('VnfPackageArtifactInfoList',
-                                            nullable=True)
+                                            nullable=True),
+        'downloading': fields.IntegerField(nullable=False, default=0)
     }
 
     def __init__(self, context=None, **kwargs):
@@ -535,6 +570,12 @@ class VnfPackage(base.TackerObject, base.TackerPersistentObject,
         return cls._from_db_object(context, cls(), db_vnf_package,
                                    expected_attrs=expected_attrs)
 
+    @base.remotable_classmethod
+    def get_by_id_with_lock(cls, context, id):
+        vnf_package = context.session.query(models.VnfPackage).filter(
+            models.VnfPackage.id == id).with_for_update().one()
+        return vnf_package
+
     @base.remotable
     def destroy(self, context):
         if not self.obj_attr_is_set('id'):
@@ -678,6 +719,22 @@ class VnfPackagesList(ovoo_base.ObjectListBase, base.TackerObject):
         return _make_vnf_packages_list(context, cls(), db_vnf_packages,
                                        expected_attrs)
 
+    @base.remotable_classmethod
+    def get_by_marker_filter(cls, context,
+            limit, marker_obj,
+            filters=None, read_deleted=None):
+        query = _vnf_packages_get_by_filters_query(context,
+                                               read_deleted=read_deleted,
+                                               filters=filters)
+        query = sqlalchemyutils.paginate_query(query,
+            model=models.VnfPackage,
+            limit=limit,
+            sorts=[['id', 'asc']],
+            marker_obj=marker_obj)
+        db_vnf_packages = query.all()
+
+        return _make_vnf_packages_list(context, cls(), db_vnf_packages)
+
     @base.remotable_classmethod
     def get_by_filters(cls, context, read_deleted=None, filters=None):
         db_vnf_packages = _vnf_package_list_by_filters(context,
