@@ -268,7 +268,8 @@ class SshIpNotFoundException(SolHttpError404):
 
 
 class CoordinateVNFExecutionFailed(SolHttpError422):
-    message = _('CoordinateVNF execution failed.')
+    title = 'Coordinate VNF execution failed'
+    # detail set in the code
 
 
 class VmRunningFailed(SolHttpError422):
@@ -383,6 +384,11 @@ class HelmOperationFailed(SolHttpError422):
     # detail set in the code
 
 
+class TerraformOperationFailed(SolHttpError422):
+    title = 'Terraform operation failed'
+    # detail set in the code
+
+
 class HelmParameterNotFound(SolHttpError400):
     message = _("Helm parameter for scale vdu %(vdu_name)s is not found.")
 
@@ -403,6 +409,10 @@ class PMJobNotExist(SolHttpError404):
     message = _("The specified PM job does not exist.")
 
 
+class PMThresholdNotExist(SolHttpError404):
+    message = _("The specified PM Threshold %(threshold_id)s does not exist.")
+
+
 class PMReportNotExist(SolHttpError404):
     message = _("The specified Performance Report does not exist.")
 
@@ -411,11 +421,20 @@ class PMJobInvalidRequest(SolHttpError400):
     message = _("Invalid request")
 
 
+class PMThresholdInvalidRequest(SolHttpError400):
+    message = _("Invalid request")
+
+
 class ResourcesOtherOperationInProgress(SolHttpError409):
     message = _("Other LCM operation of resources %(inst_id)s "
                 "is in progress.")
 
 
+# plugin base
+class MonitoringPluginClassError(SolHttpError503):
+    message = _("%(class_name)s is not a subclass of MonitoringPlugin.")
+
+
 # prometheus plugin
 class PrometheusPluginNotEnabled(SolHttpError404):
     message = _("%(name)s API is not enabled.")
