@@ -15,15 +15,17 @@ DEFAULT_ALARM_ACTIONS = ['respawn', 'log', 'log_and_kill', 'notify']
 POLICY_RESERVATION = 'tosca.policies.tacker.Reservation'
 VNF_CIRROS_CREATE_TIMEOUT = 300
 VNFC_CREATE_TIMEOUT = 600
+VNF_CIRROS_UPDATE_TIMEOUT = 300
 VNF_CIRROS_DELETE_TIMEOUT = 300
 VNF_CIRROS_DEAD_TIMEOUT = 500
 ACTIVE_SLEEP_TIME = 3
 DEAD_SLEEP_TIME = 1
 SCALE_WINDOW_SLEEP_TIME = 120
-SCALE_SLEEP_TIME = 120
+SCALE_SLEEP_TIME = 30
 NS_CREATE_TIMEOUT = 400
 NS_DELETE_TIMEOUT = 300
 NOVA_CLIENT_VERSION = 2
+CINDER_CLIENT_VERSION = 3
 VDU_MARK_UNHEALTHY_TIMEOUT = 500
 VDU_MARK_UNHEALTHY_SLEEP_TIME = 3
 VDU_AUTOHEALING_TIMEOUT = 500
@@ -38,3 +40,6 @@ LEASE_CHECK_EVENT_TIMEOUT = 300
 LEASE_CHECK_SLEEP_TIME = 3
 UUID = 'f26f181d-7891-4720-b022-b074ec1733ef'
 INVALID_UUID = 'f181d-7891-4720-b022-b074ec3ef'
+# artifact related
+ARTIFACT_PATH = 'Scripts/install.sh'
+INVALID_ARTIFACT_PATH = 'Fake_Scripts/fake_install.sh'
