@@ -17,6 +17,7 @@ import http.server
 import inspect
 import json
 import os
+import ssl
 import threading
 import time
 from urllib.parse import urlparse
@@ -70,7 +71,14 @@ def PrepareRequestHandler(manager):
                 response_body_str = open(mock_info.get('content'), 'rb').read()
             elif len(mock_body) > 0:
                 response_body_str = json.dumps(mock_body).encode('utf-8')
-                mock_headers['Content-Length'] = str(len(response_body_str))
+                if '/token' in self.path or 'v2' in self.path or (
+                        isinstance(mock_body, dict) and
+                        'v2' in mock_body.get('_links', {}).get(
+                        'vnfLcmOpOcc', {}).get('href')):
+                    pass
+                else:
+                    mock_headers['Content-Length'] = str(len(
+                        response_body_str))
 
             # Send custom header if exist
             for key, val in mock_headers.items():
@@ -87,10 +95,13 @@ def PrepareRequestHandler(manager):
                 response_headers=copy.deepcopy(mock_headers),
                 response_body=copy.deepcopy(mock_body)))
 
-            if mock_info.get('content') is None:
+            if mock_info.get('content') is None and 'v2/grants' not in path:
                 self.end_headers()
 
         def _parse_request_body(self):
+            if '/token' in self.path:
+                return {}
+
             if 'content-length' not in self.headers:
                 return {}
 
@@ -131,7 +142,8 @@ def PrepareRequestHandler(manager):
             return (status_code, mock_headers, mock_body)
 
         def do_DELETE(self):
-            raise NotImplementedError
+            self.send_response(http.HTTPStatus.NO_CONTENT)
+            self.end_headers()
 
         def do_GET(self):
             """Process GET request"""
@@ -165,8 +177,14 @@ def PrepareRequestHandler(manager):
             if self._is_match_with_list():
                 # Request is registered in our list.
                 tplUri = urlparse(self.path)
-                self._returned_callback(tplUri.path,
-                    manager._funcs_posts[tplUri.path])
+                if self.path.startswith('/server_notification'):
+                    for key in manager._funcs_posts.keys():
+                        if self.path.startswith(key):
+                            self._returned_callback(tplUri.path,
+                                manager._funcs_posts[key])
+                else:
+                    self._returned_callback(tplUri.path,
+                        manager._funcs_posts[tplUri.path])
             else:
                 # Unregistered URI is requested
                 LOG.debug('POST Recv. Unknown URL: "%s"' % self.path)
@@ -179,7 +197,28 @@ def PrepareRequestHandler(manager):
                  inspect.currentframe().f_code.co_name))
 
         def do_PUT(self):
-            raise NotImplementedError
+            """Process PUT request"""
+            LOG.debug(
+                '[Start] %s.%s()' %
+                (self.__class__.__name__,
+                 inspect.currentframe().f_code.co_name))
+
+            # URI might have trailing uuid or not.
+            if self._is_match_with_list():
+                # Request is registered in our list.
+                tplUri = urlparse(self.path)
+                self._returned_callback(tplUri.path,
+                                        manager._funcs_puts[tplUri.path])
+            else:
+                # Unregistered URI is requested
+                LOG.debug('PUT Recv. Unknown URL: "%s"' % self.path)
+                self.send_response(http.HTTPStatus.BAD_REQUEST)
+                self.end_headers()
+
+            LOG.debug(
+                '[ End ] %s.%s()' %
+                (self.__class__.__name__,
+                 inspect.currentframe().f_code.co_name))
 
     return DummyRequestHandler
 
@@ -347,6 +386,13 @@ class FakeServerManager(object):
             (self.__class__.__name__,
              inspect.currentframe().f_code.co_name))
 
+    def set_https_server(self):
+        CERTFILE = "/etc/https_server/ssl/https_server.pem"
+        context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
+        context.load_cert_chain(CERTFILE)
+        self.objHttpd.socket = context.wrap_socket(
+            self.objHttpd.socket, server_side=True)
+
     def start_server(self):
         """Start server in thread."""
         LOG.debug('[START] %s()' % inspect.currentframe().f_code.co_name)
