@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 from oslo_utils import uuidutils
 
 
@@ -59,7 +60,7 @@ def max_sample_instantiate(auth_url, bearer_token, ssl_ca_cert=None):
     vim_id_2 = uuidutils.generate_uuid()
     vim_1 = {
         "vimId": vim_id_1,
-        "vimType": "kubernetes",
+        "vimType": "ETSINFV.KUBERNETES.V_1",
         "interfaceInfo": {"endpoint": auth_url},
         "accessInfo": {
             "bearer_token": bearer_token,
@@ -68,7 +69,7 @@ def max_sample_instantiate(auth_url, bearer_token, ssl_ca_cert=None):
     }
     vim_2 = {
         "vimId": vim_id_2,
-        "vimType": "kubernetes",
+        "vimType": "ETSINFV.KUBERNETES.V_1",
         "interfaceInfo": {"endpoint": auth_url},
         "accessInfo": {
             "username": "dummy_user",
@@ -152,6 +153,15 @@ def max_sample_scale_in():
     }
 
 
+def scale_out():
+    return {
+        "type": "SCALE_OUT",
+        "aspectId": "vdu2_aspect",
+        "numberOfSteps": 1,
+        "additionalParams": {"dummy-key": "dummy-value"}
+    }
+
+
 def max_sample_heal(vnfc_ids):
     return {
         "vnfcInstanceId": vnfc_ids
@@ -161,7 +171,7 @@ def max_sample_heal(vnfc_ids):
 def min_sample_instantiate(vim_id_1):
     vim_1 = {
         "vimId": vim_id_1,
-        "vimType": "kubernetes",
+        "vimType": "ETSINFV.KUBERNETES.V_1",
     }
     return {
         "flavourId": "simple",
@@ -180,7 +190,7 @@ def min_sample_instantiate_with_vim_info(k8s_vim_info):
 
     vim_1 = {
         "vimId": uuidutils.generate_uuid(),
-        "vimType": "kubernetes",
+        "vimType": "ETSINFV.KUBERNETES.V_1",
         "accessInfo": k8s_vim_info.accessInfo,
         "interfaceInfo": k8s_vim_info.interfaceInfo
     }
@@ -211,7 +221,7 @@ def error_handling_instantiate(auth_url, bearer_token):
     vim_id_1 = uuidutils.generate_uuid()
     vim_1 = {
         "vimId": vim_id_1,
-        "vimType": "kubernetes",
+        "vimType": "ETSINFV.KUBERNETES.V_1",
         "interfaceInfo": {"endpoint": auth_url},
         "accessInfo": {
             "bearer_token": bearer_token,
@@ -250,7 +260,7 @@ def change_vnfpkg_instantiate(auth_url, bearer_token):
     vim_id_1 = uuidutils.generate_uuid()
     vim_1 = {
         "vimId": vim_id_1,
-        "vimType": "kubernetes",
+        "vimType": "ETSINFV.KUBERNETES.V_1",
         "interfaceInfo": {"endpoint": auth_url},
         "accessInfo": {
             "bearer_token": bearer_token,
@@ -318,19 +328,25 @@ def test_helm_instantiate_create(vnfd_id):
     }
 
 
-def helm_instantiate(auth_url, bearer_token, ssl_ca_cert):
-    vim_id_1 = uuidutils.generate_uuid()
-    vim_1 = {
-        "vimId": vim_id_1,
-        "vimType": "ETSINFV.HELM.V_3",
-        "interfaceInfo": {
-            "endpoint": auth_url,
-            "ssl_ca_cert": ssl_ca_cert
-        },
-        "accessInfo": {
-            "bearer_token": bearer_token
+def helm_instantiate(auth_url=None, bearer_token=None, ssl_ca_cert=None,
+                     vim_id=None):
+    if not vim_id:
+        vim_1 = {
+            "vimId": uuidutils.generate_uuid(),
+            "vimType": "ETSINFV.HELM.V_3",
+            "interfaceInfo": {
+                "endpoint": auth_url,
+                "ssl_ca_cert": ssl_ca_cert
+            },
+            "accessInfo": {
+                "bearer_token": bearer_token
+            }
+        }
+    else:
+        vim_1 = {
+            "vimId": vim_id,
+            "vimType": "ETSINFV.HELM.V_3",
         }
-    }
     return {
         "flavourId": "simple",
         "vimConnectionInfo": {
@@ -339,7 +355,8 @@ def helm_instantiate(auth_url, bearer_token, ssl_ca_cert):
         "additionalParams": {
             "helm_chart_path": "Files/kubernetes/test-chart-0.1.0.tgz",
             "helm_parameters": {
-                "service.port": 8081
+                "service.port": 8081,
+                "service.type": "NodePort"
             },
             "helm_value_names": {
                 "VDU1": {
@@ -581,6 +598,95 @@ def terminate_vnf_min():
     }
 
 
+def pm_job_external(callback_uri, inst_id, host_ip, rsc_id):
+    def pm_job(
+            callback_uri, inst_id, host_ip,
+            object_type, performance_metric,
+            sub_object_instance_id=None):
+        job = {
+            "objectType": object_type,
+            "objectInstanceIds": [inst_id],
+            "subObjectInstanceIds": ([sub_object_instance_id]
+                                     if sub_object_instance_id else []),
+            "criteria": {
+                "performanceMetric": [performance_metric],
+                "performanceMetricGroup": [],
+                "collectionPeriod": 30,
+                "reportingPeriod": 90
+            },
+            "callbackUri": callback_uri,
+            "metadata": {
+                "monitoring": {
+                    "monitorName": "prometheus",
+                    "driverType": "external",
+                    "targetsInfo": [
+                        {
+                            "prometheusHost": host_ip,
+                            "prometheusHostPort": 50022,
+                            "authInfo": {
+                                "ssh_username": "root",
+                                "ssh_password": "root"
+                            },
+                            "alertRuleConfigPath":
+                                "/tmp",
+                            "prometheusReloadApiEndpoint":
+                                "http://localhost:9990/-/reload",
+                        }
+                    ]
+                }
+            }
+        }
+        return copy.deepcopy(job)
+    return [
+        pm_job(callback_uri, inst_id, host_ip, "Vnf",
+               f"VCpuUsageMeanVnf.{inst_id}"),
+        pm_job(callback_uri, inst_id, host_ip, "Vnf",
+               f"VCpuUsagePeakVnf.{inst_id}"),
+        pm_job(callback_uri, inst_id, host_ip, "Vnf",
+               f"VMemoryUsageMeanVnf.{inst_id}"),
+        pm_job(callback_uri, inst_id, host_ip, "Vnf",
+               f"VMemoryUsagePeakVnf.{inst_id}"),
+        pm_job(callback_uri, inst_id, host_ip, "Vnf",
+               f"VDiskUsageMeanVnf.{inst_id}"),
+        pm_job(callback_uri, inst_id, host_ip, "Vnf",
+               f"VDiskUsagePeakVnf.{inst_id}"),
+        pm_job(callback_uri, inst_id, host_ip, "Vnfc",
+               f"VCpuUsageMeanVnf.{inst_id}",
+               sub_object_instance_id=rsc_id),
+        pm_job(callback_uri, inst_id, host_ip, "Vnfc",
+               f"VCpuUsagePeakVnf.{inst_id}",
+               sub_object_instance_id=rsc_id),
+        pm_job(callback_uri, inst_id, host_ip, "Vnfc",
+               f"VMemoryUsageMeanVnf.{inst_id}",
+               sub_object_instance_id=rsc_id),
+        pm_job(callback_uri, inst_id, host_ip, "Vnfc",
+               f"VMemoryUsagePeakVnf.{inst_id}",
+               sub_object_instance_id=rsc_id),
+        pm_job(callback_uri, inst_id, host_ip, "Vnfc",
+               f"VDiskUsageMeanVnf.{inst_id}",
+               sub_object_instance_id=rsc_id),
+        pm_job(callback_uri, inst_id, host_ip, "Vnfc",
+               f"VDiskUsagePeakVnf.{inst_id}",
+               sub_object_instance_id=rsc_id),
+        pm_job(callback_uri, inst_id, host_ip, "VnfIntCp",
+               "ByteIncomingVnfIntCp", sub_object_instance_id="eth0"),
+        pm_job(callback_uri, inst_id, host_ip, "VnfIntCp",
+               "PacketIncomingVnfIntCp", sub_object_instance_id="eth0"),
+        pm_job(callback_uri, inst_id, host_ip, "VnfIntCp",
+               "ByteOutgoingVnfIntCp", sub_object_instance_id="eth0"),
+        pm_job(callback_uri, inst_id, host_ip, "VnfIntCp",
+               "PacketOutgoingVnfIntCp", sub_object_instance_id="eth0"),
+        pm_job(callback_uri, inst_id, host_ip, "VnfExtCp",
+               "ByteIncomingVnfExtCp", sub_object_instance_id="eth1"),
+        pm_job(callback_uri, inst_id, host_ip, "VnfExtCp",
+               "PacketIncomingVnfExtCp", sub_object_instance_id="eth1"),
+        pm_job(callback_uri, inst_id, host_ip, "VnfExtCp",
+               "ByteOutgoingVnfExtCp", sub_object_instance_id="eth1"),
+        pm_job(callback_uri, inst_id, host_ip, "VnfExtCp",
+               "PacketOutgoingVnfExtCp", sub_object_instance_id="eth1")
+    ]
+
+
 def pm_job_min(callback_uri, inst_id, host_ip):
     return {
         "objectType": "Vnf",
@@ -659,6 +765,113 @@ def update_pm_job(callback_uri):
     }
 
 
+def pm_threshold_min(
+        callback_uri, inst_id, host_ip,
+        objectType="Vnf",
+        sub_object_instance_id=None,
+        p_metric=None,
+        thresholdValue=55,
+        hysteresis=30):
+    metric = f"VCpuUsageMeanVnf.{inst_id}"
+    if p_metric:
+        metric = f"{p_metric}"
+    return {
+        "objectType": objectType,
+        "objectInstanceId": inst_id,
+        "subObjectInstanceIds": ([sub_object_instance_id]
+                                 if sub_object_instance_id else []),
+        "criteria": {
+            "performanceMetric": metric,
+            "thresholdType": "SIMPLE",
+            "simpleThresholdDetails": {
+                "thresholdValue": thresholdValue,
+                "hysteresis": hysteresis
+            }
+        },
+        "callbackUri": callback_uri,
+        "metadata": {
+            "monitoring": {
+                "monitorName": "prometheus",
+                "driverType": "external",
+                "targetsInfo": [
+                    {
+                        "prometheusHost": host_ip,
+                        "prometheusHostPort": 50022,
+                        "authInfo": {
+                            "ssh_username": "root",
+                            "ssh_password": "root"
+                        },
+                        "alertRuleConfigPath":
+                            "/tmp",
+                        "prometheusReloadApiEndpoint":
+                            "http://localhost:9990/-/reload"
+                    }
+                ]
+            }
+        }
+    }
+
+
+def pm_threshold_max(
+        callback_uri, inst_id, host_ip,
+        objectType="Vnf",
+        sub_object_instance_id=None,
+        p_metric=None,
+        thresholdValue=55,
+        hysteresis=30):
+    metric = f"VCpuUsageMeanVnf.{inst_id}"
+    if p_metric:
+        metric = f"{p_metric}"
+    return {
+        "objectType": objectType,
+        "objectInstanceId": inst_id,
+        "subObjectInstanceIds": ([sub_object_instance_id]
+                                 if sub_object_instance_id else []),
+        "criteria": {
+            "performanceMetric": metric,
+            "thresholdType": "SIMPLE",
+            "simpleThresholdDetails": {
+                "thresholdValue": thresholdValue,
+                "hysteresis": hysteresis
+            }
+        },
+        "callbackUri": callback_uri,
+        "authentication": {
+            "authType": ["BASIC"],
+            "paramsBasic": {
+                "userName": "test",
+                "password": "test"
+            }
+        },
+        "metadata": {
+            "monitoring": {
+                "monitorName": "prometheus",
+                "driverType": "external",
+                "targetsInfo": [
+                    {
+                        "prometheusHost": host_ip,
+                        "prometheusHostPort": 50022,
+                        "authInfo": {
+                            "ssh_username": "root",
+                            "ssh_password": "root"
+                        },
+                        "alertRuleConfigPath":
+                            "/tmp",
+                        "prometheusReloadApiEndpoint":
+                            "http://localhost:9990/-/reload"
+                    }
+                ]
+            }
+        }
+    }
+
+
+def update_pm_threshold(callback_uri):
+    return {
+        "callbackUri": callback_uri
+    }
+
+
 def pm_event(job_id, inst_id):
     return {
         "receiver": "receiver",
@@ -698,6 +911,51 @@ def pm_event(job_id, inst_id):
     }
 
 
+def pm_threshold(threshold_id, inst_id,
+                 sub_inst_id=None, value=99, p_metric=None):
+    metric = f"VCpuUsageMeanVnf.{inst_id}"
+    if p_metric:
+        metric = f"{p_metric}"
+    # This data simulates the complete request body sent by alertmanager.
+    return {
+        "receiver": "receiver",
+        "status": "firing",
+        "alerts": [
+            {
+                "status": "firing",
+                "labels": {
+                    "receiver_type": "tacker",
+                    "function_type": "vnfpm_threshold",
+                    "threshold_id": threshold_id,
+                    "metric": metric,
+                    "object_instance_id": inst_id,
+                    "sub_object_instance_id": sub_inst_id
+                },
+                "annotations": {
+                    "value": value,
+                },
+                "startsAt": "2022-12-15T23:47:36.453Z",
+                "endsAt": "0001-01-01T00:00:00Z",
+                "generatorURL": "http://controller147:9090/graph?g0.expr=up%7B"
+                                "job%3D%22node%22%7D+%3D%3D+0&g0.tab=1",
+                "fingerprint": "5ef77f1f8a3ecb8d"
+            }
+        ],
+        "groupLabels": {},
+        "commonLabels": {
+            "alertname": "NodeInstanceDown",
+            "job": "node"
+        },
+        "commonAnnotations": {
+            "description": "sample"
+        },
+        "externalURL": "http://controller147:9093",
+        "version": "4",
+        "groupKey": "{}:{}",
+        "truncatedAlerts": 0
+    }
+
+
 def prometheus_auto_scaling_alert(inst_id):
     return {
         "receiver": "receiver",
@@ -732,3 +990,87 @@ def prometheus_auto_scaling_alert(inst_id):
         "groupKey": "{}:{}",
         "truncatedAlerts": 0
     }
+
+
+def prometheus_auto_healing_alert(inst_id, vnfc_info_id):
+    return {
+        "receiver": "receiver",
+        "status": "firing",
+        "alerts": [{
+            "status": "firing",
+            "labels": {
+                "receiver_type": "tacker",
+                "function_type": "auto_heal",
+                "vnf_instance_id": inst_id,
+                "vnfc_info_id": vnfc_info_id
+            },
+            "annotations": {
+            },
+            "startsAt": "2022-06-21T23:47:36.453Z",
+            "endsAt": "0001-01-01T00:00:00Z",
+            "generatorURL": "http://controller147:9090/graph?g0.expr="
+                            "up%7Bjob%3D%22node%22%7D+%3D%3D+0&g0.tab=1",
+            "fingerprint": "5ef77f1f8a3ecb8d"
+        }],
+        "groupLabels": {},
+        "commonLabels": {
+            "alertname": "NodeInstanceDown",
+            "job": "node"
+        },
+        "commonAnnotations": {
+            "description": "sample"
+        },
+        "externalURL": "http://controller147:9093",
+        "version": "4",
+        "groupKey": "{}:{}",
+        "truncatedAlerts": 0
+    }
+
+
+def test_cnf_update_create(vnfd_id):
+    return {
+        "vnfdId": vnfd_id,
+        "vnfInstanceName": "container_update_multi_kinds",
+        "vnfInstanceDescription": "container update multi kinds"
+    }
+
+
+def test_cnf_update_instantiate(vim_id):
+    return {
+        "flavourId": "simple",
+        "vimConnectionInfo": {
+            "vim1": {
+                "vimId": vim_id,
+                "vimType": "ETSINFV.KUBERNETES.V_1"
+            }
+        },
+        "additionalParams": {
+            "lcm-kubernetes-def-files": [
+                "Files/kubernetes/configmap_1.yaml",
+                "Files/kubernetes/deployment.yaml",
+                "Files/kubernetes/pod_env.yaml",
+                "Files/kubernetes/pod_volume.yaml",
+                "Files/kubernetes/replicaset.yaml",
+                "Files/kubernetes/secret_1.yaml",
+                "Files/kubernetes/configmap_3.yaml",
+                "Files/kubernetes/pod_env_2.yaml",
+                "Files/kubernetes/pod_volume_2.yaml",
+                "Files/kubernetes/daemonset.yaml",
+                "Files/kubernetes/deployment_2.yaml",
+                "Files/kubernetes/secret_3.yaml"
+            ]
+        }
+    }
+
+
+def test_cnf_update_modify(vnfd_id):
+    return {
+        "vnfdId": vnfd_id,
+        "vnfInstanceName": "modify_vnf_after",
+        "metadata": {
+            "configmap_secret_paths": [
+                "Files/kubernetes/configmap_2.yaml",
+                "Files/kubernetes/secret_2.yaml"
+            ]
+        }
+    }
