@@ -20,6 +20,8 @@ from tacker.objects import fields
 from tacker.tests.functional.sol_kubernetes_v2 import base_v2
 from tacker.tests.functional.sol_kubernetes_v2 import paramgen
 
+WAIT_NOTIFICATION_TIME = 5
+
 
 @ddt.ddt
 class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
@@ -32,17 +34,17 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
 
         test_instantiate_cnf_resources_path = os.path.join(
             cur_dir, "samples/test_instantiate_cnf_resources")
-        cls.vnf_pkg_1, cls.vnfd_id_1 = cls.create_vnf_package(
+        cls.cnf_pkg, cls.cnf_vnfd_id = cls.create_vnf_package(
             test_instantiate_cnf_resources_path)
 
     @classmethod
     def tearDownClass(cls):
         super(VnfPmTest, cls).tearDownClass()
-        cls.delete_vnf_package(cls.vnf_pkg_1)
+        cls.delete_vnf_package(cls.cnf_pkg)
 
     def setUp(self):
         super(VnfPmTest, self).setUp()
-        base_v2.FAKE_SERVER_MANAGER.set_callback(
+        self.set_server_callback(
             'PUT', "/-/reload", status_code=202,
             response_headers={"Content-Type": "text/plain"})
 
@@ -75,7 +77,7 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         # NOTE: extensions and vnfConfigurableProperties are omitted
         # because they are commented out in etsi_nfv_sol001.
         create_req = paramgen.pm_instantiate_cnf_resources_create(
-            self.vnfd_id_1)
+            self.cnf_vnfd_id)
         resp, body = self.create_vnf_instance(create_req)
         self.assertEqual(201, resp.status_code)
         inst_id = body['id']
@@ -100,10 +102,10 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
             'callbackUri',
             '_links'
         ]
-        callback_url = os.path.join(base_v2.MOCK_NOTIFY_CALLBACK_URL,
+        callback_url = os.path.join(self.get_notify_callback_url(),
                                     self._testMethodName)
         callback_uri = ('http://localhost:'
-                        f'{base_v2.FAKE_SERVER_MANAGER.SERVER_PORT}'
+                        f'{self.get_server_port()}'
                         f'{callback_url}')
         sub_req = paramgen.pm_job_min(
             callback_uri, inst_id, self.fake_prometheus_ip)
@@ -116,16 +118,14 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         pm_job_id = body.get('id')
 
         # 4. PMJob-Update
-        callback_url = os.path.join(base_v2.MOCK_NOTIFY_CALLBACK_URL,
+        callback_url = os.path.join(self.get_notify_callback_url(),
                                     self._testMethodName)
-        callback_url = callback_url + '_1'
+        callback_url = f"{callback_url}_1"
         callback_uri = ('http://localhost:'
-                        f'{base_v2.FAKE_SERVER_MANAGER.SERVER_PORT}'
+                        f'{self.get_server_port()}'
                         f'{callback_url}')
-        base_v2.FAKE_SERVER_MANAGER.set_callback(
-            'GET', callback_url, status_code=204)
-        base_v2.FAKE_SERVER_MANAGER.set_callback(
-            'POST', callback_url, status_code=204)
+        self.set_server_callback('GET', callback_url, status_code=204)
+        self.set_server_callback('POST', callback_url, status_code=204)
         update_req = paramgen.update_pm_job(callback_uri)
         resp, body = self.update_pm_job(pm_job_id, update_req)
         self.assertEqual(200, resp.status_code)
@@ -137,7 +137,7 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         sub_req = paramgen.pm_event(pm_job_id, inst_id)
         resp, body = self.create_pm_event(sub_req)
         self.assertEqual(204, resp.status_code)
-        time.sleep(5)
+        time.sleep(WAIT_NOTIFICATION_TIME)
         self._check_notification(
             callback_url, 'PerformanceInformationAvailableNotification')
 
@@ -177,10 +177,6 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         lcmocc_id = os.path.basename(resp.headers['Location'])
         self.wait_lcmocc_complete(lcmocc_id)
 
-        # wait a bit because there is a bit time lag between lcmocc DB
-        # update and terminate completion.
-        time.sleep(10)
-
         # check instantiationState of VNF
         resp, body = self.show_vnf_instance(inst_id)
         self.assertEqual(200, resp.status_code)
@@ -188,7 +184,7 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
                          body['instantiationState'])
 
         # 11. LCM-Delete: Delete a VNF instance
-        resp, body = self.delete_vnf_instance(inst_id)
+        resp, body = self.exec_lcm_operation(self.delete_vnf_instance, inst_id)
         self.assertEqual(204, resp.status_code)
 
         # check deletion of VNF instance
@@ -223,7 +219,8 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         # 1. LCM-Create: Create a new VNF instance resource
         # NOTE: extensions and vnfConfigurableProperties are omitted
         # because they are commented out in etsi_nfv_sol001.
-        create_req = paramgen.instantiate_cnf_resources_create(self.vnfd_id_1)
+        create_req = paramgen.instantiate_cnf_resources_create(
+            self.cnf_vnfd_id)
         resp, body = self.create_vnf_instance(create_req)
         self.assertEqual(201, resp.status_code)
         inst_id = body['id']
@@ -248,10 +245,10 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
             'callbackUri',
             '_links'
         ]
-        callback_url = os.path.join(base_v2.MOCK_NOTIFY_CALLBACK_URL,
+        callback_url = os.path.join(self.get_notify_callback_url(),
                                     self._testMethodName)
         callback_uri = ('http://localhost:'
-                        f'{base_v2.FAKE_SERVER_MANAGER.SERVER_PORT}'
+                        f'{self.get_server_port()}'
                         f'{callback_url}')
         sub_req = paramgen.pm_job_max(
             callback_uri, inst_id, self.fake_prometheus_ip)
@@ -264,16 +261,14 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         pm_job_id = body.get('id')
 
         # 4. PMJob-Update
-        callback_url = os.path.join(base_v2.MOCK_NOTIFY_CALLBACK_URL,
+        callback_url = os.path.join(self.get_notify_callback_url(),
                                     self._testMethodName)
-        callback_url = callback_url + '_1'
+        callback_url = f"{callback_url}_1"
         callback_uri = ('http://localhost:'
-                        f'{base_v2.FAKE_SERVER_MANAGER.SERVER_PORT}'
+                        f'{self.get_server_port()}'
                         f'{callback_url}')
-        base_v2.FAKE_SERVER_MANAGER.set_callback(
-            'GET', callback_url, status_code=204)
-        base_v2.FAKE_SERVER_MANAGER.set_callback(
-            'POST', callback_url, status_code=204)
+        self.set_server_callback('GET', callback_url, status_code=204)
+        self.set_server_callback('POST', callback_url, status_code=204)
         update_req = paramgen.update_pm_job(callback_uri)
         resp, body = self.update_pm_job(pm_job_id, update_req)
         self.assertEqual(200, resp.status_code)
@@ -285,7 +280,7 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         sub_req = paramgen.pm_event(pm_job_id, inst_id)
         resp, body = self.create_pm_event(sub_req)
         self.assertEqual(204, resp.status_code)
-        time.sleep(5)
+        time.sleep(WAIT_NOTIFICATION_TIME)
         self._check_notification(
             callback_url, 'PerformanceInformationAvailableNotification')
 
@@ -326,10 +321,6 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         lcmocc_id = os.path.basename(resp.headers['Location'])
         self.wait_lcmocc_complete(lcmocc_id)
 
-        # wait a bit because there is a bit time lag between lcmocc DB
-        # update and terminate completion.
-        time.sleep(10)
-
         # check instantiationState of VNF
         resp, body = self.show_vnf_instance(inst_id)
         self.assertEqual(200, resp.status_code)
@@ -337,7 +328,7 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
                          body['instantiationState'])
 
         # 11. LCM-Delete: Delete a VNF instance
-        resp, body = self.delete_vnf_instance(inst_id)
+        resp, body = self.exec_lcm_operation(self.delete_vnf_instance, inst_id)
         self.assertEqual(204, resp.status_code)
 
         # check deletion of VNF instance
@@ -359,7 +350,7 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         """
         # 1. LCM-Create: Create a new VNF instance resource
         create_req = paramgen.pm_instantiate_cnf_resources_create(
-            self.vnfd_id_1)
+            self.cnf_vnfd_id)
         resp, body = self.create_vnf_instance(create_req)
         self.assertEqual(201, resp.status_code)
         inst_id = body['id']
@@ -387,10 +378,10 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
             'callbackUri',
             '_links'
         ]
-        callback_url = os.path.join(base_v2.MOCK_NOTIFY_CALLBACK_URL,
+        callback_url = os.path.join(self.get_notify_callback_url(),
                                     self._testMethodName)
         callback_uri = ('http://localhost:'
-                        f'{base_v2.FAKE_SERVER_MANAGER.SERVER_PORT}'
+                        f'{self.get_server_port()}'
                         f'{callback_url}')
         pm_job_list = paramgen.pm_job_external(
             callback_uri, inst_id, self.fake_prometheus_ip, rsc)
@@ -417,10 +408,6 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
         lcmocc_id = os.path.basename(resp.headers['Location'])
         self.wait_lcmocc_complete(lcmocc_id)
 
-        # wait a bit because there is a bit time lag between lcmocc DB
-        # update and terminate completion.
-        time.sleep(10)
-
         # check instantiationState of VNF
         resp, body = self.show_vnf_instance(inst_id)
         self.assertEqual(200, resp.status_code)
@@ -428,7 +415,7 @@ class VnfPmTest(base_v2.BaseVnfLcmKubernetesV2Test):
                          body['instantiationState'])
 
         # 6. LCM-Delete: Delete a VNF instance
-        resp, body = self.delete_vnf_instance(inst_id)
+        resp, body = self.exec_lcm_operation(self.delete_vnf_instance, inst_id)
         self.assertEqual(204, resp.status_code)
 
         # check deletion of VNF instance
