@@ -14,10 +14,11 @@
 #    under the License.
 
 import os
+from unittest import mock
+
+from oslo_config import cfg
 
 from keystoneauth1 import exceptions
-import mock
-from oslo_config import cfg
 
 from tacker import context as t_context
 from tacker.extensions import nfvo
@@ -38,13 +39,13 @@ OPTS = [cfg.StrOpt('user_domain_id',
                    default='default',
                    help='User Name'),
         cfg.StrOpt('user_domain_name',
-                   default='default',
+                   default='Default',
                    help='Use Domain Name'),
         cfg.StrOpt('project_name',
                    default='default',
                    help='Project Name'),
         cfg.StrOpt('project_domain_name',
-                   default='default',
+                   default='Default',
                    help='Project Domain Name'),
         cfg.StrOpt('auth_url',
                    default='http://localhost/identity/v3',
@@ -54,6 +55,38 @@ cfg.CONF.register_opts(OPTS, 'keystone_authtoken')
 CONF = cfg.CONF
 
 
+def get_mock_conf_key_effect():
+    def mock_conf_key_effect(name):
+        if name == 'keystone_authtoken':
+            return MockConfig(conf=None)
+        elif name == 'ext_oauth2_auth':
+            return MockConfig(
+                conf={
+                    'use_ext_oauth2_auth': True,
+                    'token_endpoint': 'http://demo/token_endpoint',
+                    'auth_method': 'client_secret_post',
+                    'client_id': 'client_id',
+                    'client_secret': 'client_secret',
+                    'scope': 'client_secret'
+                })
+        elif name == 'key_manager':
+            conf = {
+                'api_class': 'tacker.keymgr.barbican_key_manager'
+                             '.BarbicanKeyManager',
+                'barbican_version': 'v1',
+                'barbican_endpoint': 'http://test/barbican'
+            }
+            return MockConfig(conf=conf)
+        elif name == 'vim_keys':
+            return MockConfig(
+                conf={
+                    'use_barbican': True
+                })
+        else:
+            return cfg.CONF._get(name)
+    return mock_conf_key_effect
+
+
 class FakeKeystone(mock.Mock):
     pass
 
@@ -74,6 +107,21 @@ class mock_dict(dict):
     __delattr__ = dict.__delitem__
 
 
+class MockConfig(object):
+    def __init__(self, conf=None):
+        self.conf = conf
+
+    def __getattr__(self, name):
+        if not self.conf:
+            raise cfg.NoSuchOptError('not found %s' % name)
+        if name not in self.conf:
+            raise cfg.NoSuchOptError('not found %s' % name)
+        return self.conf.get(name)
+
+    def __contains__(self, key):
+        return key in self.conf
+
+
 class TestOpenstack_Driver(base.TestCase):
     def setUp(self):
         super(TestOpenstack_Driver, self).setUp()
@@ -107,35 +155,33 @@ class TestOpenstack_Driver(base.TestCase):
                 'openstack', 'auth_url': 'http://localhost/identity',
                 'auth_cred': {'username': 'test_user',
                               'password': 'test_password',
-                              'user_domain_name': 'default',
+                              'user_domain_name': 'Default',
                               'cert_verify': 'True',
                               'auth_url': 'http://localhost/identity'},
                 'name': 'VIM0',
                 'vim_project': {'name': 'test_project',
-                                'project_domain_name': 'default'}}
+                                'project_domain_name': 'Default'}}
 
     def get_vim_obj_barbican(self):
         return {'id': '6261579e-d6f3-49ad-8bc3-a9cb974778ff', 'type':
                 'openstack', 'auth_url': 'http://localhost/identity',
                 'auth_cred': {'username': 'test_user',
                               'password': 'test_password',
-                              'user_domain_name': 'default',
+                              'user_domain_name': 'Default',
                               'key_type': 'barbican_key',
                               'secret_uuid': 'fake-secret-uuid',
                               'cert_verify': 'True',
                               'auth_url': 'http://localhost/identity'},
                 'name': 'VIM0',
                 'vim_project': {'name': 'test_project',
-                                'project_domain_name': 'default'}}
+                                'project_domain_name': 'Default'}}
 
     def test_register_keystone_v3(self):
-        regions = [mock_dict({'id': 'RegionOne'})]
-        attrs = {'regions.list.return_value': regions}
-        keystone_version = 'v3'
+        regions = mock_dict(regions=[{'id': 'RegionOne'}])
+        attrs = {'get.return_value.json.return_value': regions}
         mock_ks_client = mock.Mock(**attrs)
-        self.keystone.get_version.return_value = keystone_version
         self._test_register_vim(self.vim_obj, mock_ks_client)
-        mock_ks_client.regions.list.assert_called_once_with()
+        mock_ks_client.get.assert_called_once_with('/v3/regions')
         self.keystone.initialize_client.assert_called_once_with(
             **self.auth_obj)
 
@@ -195,11 +241,11 @@ class TestOpenstack_Driver(base.TestCase):
                          'fake-secret-uuid')
 
     def test_register_vim_invalid_auth(self):
-        attrs = {'regions.list.side_effect': exceptions.Unauthorized}
+        attrs = {'get.side_effect': exceptions.Unauthorized}
         self._test_register_vim_auth(attrs)
 
     def test_register_vim_missing_auth(self):
-        attrs = {'regions.list.side_effect': exceptions.BadRequest}
+        attrs = {'get.side_effect': exceptions.BadRequest}
         self._test_register_vim_auth(attrs)
 
     def _test_register_vim_auth(self, attrs):
@@ -210,7 +256,7 @@ class TestOpenstack_Driver(base.TestCase):
         self.assertRaises(nfvo.VimUnauthorizedException,
                           self.openstack_driver.register_vim,
                           self.vim_obj)
-        mock_ks_client.regions.list.assert_called_once_with()
+        mock_ks_client.get.assert_called_once_with('/v3/regions')
         self.keystone.initialize_client.assert_called_once_with(
             **self.auth_obj)
 
@@ -257,3 +303,31 @@ class TestOpenstack_Driver(base.TestCase):
         self.assertRaises(nfvo.VimGetResourceNotFoundException,
                           self.openstack_driver.get_vim_resource_id,
                           self.vim_obj, resource_type, resource_name)
+
+    @mock.patch('oslo_config.cfg.ConfigOpts.__getattr__')
+    def test_deregister_vim_barbican_ext_oauth2_auth(self, mock_get_conf_key):
+        mock_get_conf_key.side_effect = get_mock_conf_key_effect()
+        self.keymgr.delete.return_value = None
+        vim_obj = self.get_vim_obj_barbican()
+        self.openstack_driver.deregister_vim(vim_obj)
+
+    @mock.patch('oslo_config.cfg.ConfigOpts.__getattr__')
+    def test_encode_vim_auth_barbican_ext_oauth2_auth(self, mock_get_conf_key):
+        mock_get_conf_key.side_effect = get_mock_conf_key_effect()
+
+        fernet_attrs = {'encrypt.return_value': 'encrypted_password'}
+        mock_fernet_obj = mock.Mock(**fernet_attrs)
+        mock_fernet_key = 'test_fernet_key'
+        self.keymgr.store.return_value = 'fake-secret-uuid'
+        self.keystone.create_fernet_key.return_value = (mock_fernet_key,
+                                                        mock_fernet_obj)
+
+        vim_obj = self.get_vim_obj()
+        self.openstack_driver.encode_vim_auth(
+            vim_obj['id'], vim_obj['auth_cred'])
+
+        mock_fernet_obj.encrypt.assert_called_once_with(mock.ANY)
+        self.assertEqual(vim_obj['auth_cred']['key_type'],
+                         'barbican_key')
+        self.assertEqual(vim_obj['auth_cred']['secret_uuid'],
+                         'fake-secret-uuid')
