@@ -31,6 +31,7 @@ from oslo_utils import excutils
 from oslo_utils import timeutils
 from toscaparser import tosca_template
 
+from tacker._i18n import _
 from tacker.common import driver_manager
 from tacker.common import exceptions
 from tacker.common import log
@@ -41,6 +42,7 @@ from tacker import manager
 from tacker import objects
 from tacker.objects import fields
 from tacker.objects.fields import ErrorPoint as EP
+from tacker.tosca import utils as toscautils
 from tacker.vnflcm import abstract_driver
 from tacker.vnflcm import utils as vnflcm_utils
 
@@ -355,7 +357,7 @@ class VnfLcmDriver(abstract_driver.VnfInstanceAbstractDriver):
             vnf_package_path=vnf_package_path)
 
         # save the vnf resources in the db
-        for _, resources in vnf_resources.items():
+        for _i, resources in vnf_resources.items():
             for vnf_resource in resources:
                 vnf_resource.create()
 
@@ -1419,7 +1421,8 @@ class VnfLcmDriver(abstract_driver.VnfInstanceAbstractDriver):
                 vnf_instance.vnfd_id,
                 vnf_instance.instantiated_vnf_info.flavour_id)
             tosca = tosca_template.ToscaTemplate(
-                parsed_params={}, a_file=False, yaml_dict_tpl=vnfd_dict)
+                parsed_params={}, a_file=False, yaml_dict_tpl=vnfd_dict,
+                local_defs=toscautils.tosca_tmpl_local_defs())
             extract_policy_infos = vnflcm_utils.get_extract_policy_infos(tosca)
             policy['vdu_defs'] = vnflcm_utils.get_target_vdu_def_dict(
                 extract_policy_infos=extract_policy_infos,
@@ -1488,7 +1491,7 @@ class VnfLcmDriver(abstract_driver.VnfInstanceAbstractDriver):
                 region_name=vim_connection_info.access_info.get('region_name')
             )
         else:
-            for _ in range(scale_vnf_request.number_of_steps):
+            for _i in range(scale_vnf_request.number_of_steps):
                 last_event_id = self._vnf_manager.invoke(
                     vim_connection_info.vim_type,
                     'scale',
